' TIME MACHINE
' WBA Consulting
' Andrew Williams
' (c)2013
'
'  Code emulates a Time Machine Prop for a Church Christmas Kids Play.
'  Uses WAV files for sound effects, drives a VGA display, and lights some LEDs
'
' Version Info:
' 011   Copied from demo 008 code. Added jumpyear PUB
' 013   Demo with new Jumpyear method
' 014   Cleaned up code and added xbee
' 015   setup xBee rx buffer and case statement
' 016   Final cleanup  
'

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

    x_tiles = 16
    y_tiles = 12

'WAV Player
    lPin = 26
    rPin = 27

    doPin = 22
    clkPin = 23
    diPin = 24
    csPin = 25

    wpPin = -1
    cdPin = -1

VAR
  long  bitmap[3072+64] ' Must be aligned on 32 byte boundary
  long  colors[64]
  word  screen[x_tiles * y_tiles]

  long bitmapAddr
  byte numStr[5]

OBJ
  vga   : "vga"
  gr    : "graphics"
  wav   : "V2-WAV_DACEngine.spin"
  xb    : "fullduplexserial"

PUB start | DataIn

'VGA Initilization
  StartVGA

'WAV Player Initilization 
   wav.begin(lPin, rPin, doPin, clkPin, diPin, csPin, wpPin, cdPin)

'xBee Initialization
  xb.start(16, 17, 0, 9600)           ' DO, DI, mode, baud

  dira[0..7]~~
  OUTA[0..7] := %11111100

' ~~~~~~~~~~~~~~~~~~~~

  WAITMS(2000)                     
  SYSREADY

' start of loop
 
  Repeat
    DataIn := XB.RxTime(3000)         ' Wait for byte with timeout
    If DataIn == "!"
       DataIn := XB.RxTime(3000)
       case DataIn
         "1" :
          JUMPYEAR(2013, 1978, 1, 300) 
         "2" :
          JUMPYEAR(1978, 1985, 1, 1500) 
         "3" :
          JUMPYEAR(1985, 1959, 1, 460)
         "4" :
          JUMPYEAR(1959, 1998, 1, 300)
         "5" :
          JUMPYEAR(1998, 1944, 1, 200)
         "6" :
          JUMPYEAR(1944, 0, 3, 18)
         "7" :
          JUMPYEAR(0, 1944, 3, 18)
         "8" :
          JUMPYEAR(1944, 2013, 1, 170)
         "9" :
           gr.clear
           sysready
         "0" :
          dataentry

       xb.rxflush                         'clear receive buffer

PUB SYSREADY

  gr.textmode(3,3,6,5)
  gr.colorwidth(1,3)
  gr.text(-40,80,string("SYS READY"))

  gr.textmode(2,2,6,5)
  gr.colorwidth(2,2)
  gr.text(-70,-70,string("HO-2000"))

PUB JUMPSTANDBY

  gr.clear
  gr.textmode(3,3,6,5)
  gr.colorwidth(1,3)
  gr.text(0,75,string("12 / 24"))

  gr.textmode(9,9,6,5)
  gr.colorwidth(1,3) 
  gr.text(0,10,string("----"))

  gr.textmode(2,2,6,5)
  gr.colorwidth(2,2)
  gr.text(-70,-70,string("HO-2000"))
 
  gr.textmode(2,2,6,5)
  gr.colorwidth(3, 2)
  gr.text(50, -70, string("12/24/----"))

PUB DATAENTRY

    gr.clear
    SYSREADY
    wav.play(string("keys.wav"))  
    gr.textmode(4,4,6,0)
    gr.colorwidth(1,3) 
    gr.text(-70,10,string("#>1"))
    WAITMS(750)                                               
    gr.text(-70,10,string("#>12"))
    WAITMS(750)                                                   

    gr.clear
    SYSREADY
    gr.textmode(2,2,6,5)
    gr.colorwidth(3, 2)
    gr.text(50, -70, string("12/--/----"))

    gr.colorwidth(1,3)
    gr.textmode(4,4,6,0)
    WAITMS(250)                                                
    gr.text(-70,10,string("#>2"))
    WAITMS(250)                                               
    gr.text(-70,10,string("#>24"))
    WAITMS(750)                                                
    
    gr.clear
    SYSREADY
    gr.textmode(2,2,6,5)
    gr.colorwidth(3, 2)
    gr.text(50, -70, string("12/24/----"))

    gr.textmode(4,4,6,0)
    gr.colorwidth(1,3) 
    gr.text(-70,10,string("#> "))

PUB Jumpyear(depyear, arryear, j, delayadj) | i

   wav.play(string("SFX.wav"))               ' Play 24 second long "time jump" sound effect
  WAITMS(2000)                               ' Let SFX get going
  JUMPSTANDBY                                ' Show time machine activation/standby display
  WAITMS(2000)                               ' and leave up for 2 seconds before counting
    
' Count from departure year to arrival year

   Repeat i from depyear to arryear step j
        num2str(i)
        OUTA[0..7] := %10101011
        gr.textmode(9,9,6,5)
        gr.colorwidth(1,3)
        gr.text(0,10,@numStr)
        WAITMS(delayadj)                                
        OUTA[0..7] := %11010111
        gr.colorwidth(0,8)
        gr.box(-120,-30,240,81)

   OUTA[0..7] := %11111110         
' Flash arrival (present) year
   repeat 8
      gr.colorwidth(0,8)
      gr.box(-120,-30,240,81)
      WAITMS(250)                                        
      gr.textmode(9,9,6,5)
      gr.colorwidth(1,8)
      num2str(arryear) 
      gr.text(0,10,@numStr)
      WAITMS(1000)

  OUTA[0..7] := %10001100                                          


 
PUB num2str(value) | given ' For numbers from 100 to 9999
  given := value
  if value < 1000
    value *= 10
    
  numStr[0] := value / 1000  + "0"
  value //= 1000
  numStr[1] := value / 100 + "0"
  value //= 100
  numStr[2] := value / 10 + "0"
  value //= 10
  numStr[3] := value + "0"
  numStr[4] := 0

  if given < 1000
    numStr[3] := 0

PUB StartVGA | i, dx, dy, c

  bitmapAddr := (@bitmap + $3F) & $FFFF_FFC0 ' Bitmap address must start on a 32 byte boundary 

  'start vga
  vga_videobase := @screen
  vga_colorbase := @colors
  vga.start(@vgaparams)

  'init colors
  repeat i from 0 to 63
    colors[i] := $28_48_FF_00 ' teal, purple, white, black

  'init tile screen
  repeat dx from 0 to vga_hc - 1
    repeat dy from 0 to vga_vc - 1
      screen[dy * vga_hc + dx] := bitmapAddr >> 6 + dy + dx * vga_vc         '+ ((dy & $3F) << 10)

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 128, 96, bitmapAddr)


PUB WAITMS(ms)  'wait time in milliseconds
   waitcnt(clkfreq / 1000 * mS + cnt) 

DAT

vgaparams               long    0               'status
                        long    1               'enable
                        long    %001_111        'pins
                        long    %0011           'mode
vga_videobase           long    0               'videobase
vga_colorbase           long    0               'colorbase
vga_hc                  long    x_tiles         'hc
vga_vc                  long    y_tiles         'vc
                        long    2               'hx        //RJA increased from 1X to 2X
                        long    2               'vx        //RJA increased from 1X to 2X 
                        long    0               'ho
                        long    0               'vo
                        long    512             'hd
                        long    16              'hf
                        long    96              'hs
                        long    48              'hb
                        long    380+16          'vd         //RJA added 16
                        long    11              'vf
                        long    2               'vs
                        long    31+16           'vb       //RJA subtracted 16
                        long    20_000_000      'rate